#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include "emo.h"

#define SPHERE  1
#define SIMPLEX 2

// random points
int main(int argc, char **argv) {
  int *filter, *seed, row, col, i, j, k, front_type, run;
  double *data, *norm, *min, *max, s;
  EMO_Plot plot;
  EMO_Rand rnd;
  EMO_VPath vpath;

  if(argc != 6) {
    printf("Syntax: %s rows columns front_type[1:sphere,2:simplex] runs resolution\n", argv[0]);
    exit(1);
  }

  row = atoi(argv[1]);
  col = atoi(argv[2]);
  front_type = atoi(argv[3]);
  run = atoi(argv[4]);

  EMO_Rand_alloc(&rnd, NULL, 17);

  EMO_VPath_alloc(&vpath, atoi(argv[5]), row, col);

  EMO_Plot_alloc(&plot, "x11", "test", "", row, col, 1);

  data = (double *) malloc(sizeof(double) * row * col);
  norm = (double *) malloc(sizeof(double) * row * col);
  filter = (int *) malloc(sizeof(int) * row);
  seed = (int *) malloc(sizeof(int) * col);
  min = (double *) malloc(sizeof(double) * col);
  max = (double *) malloc(sizeof(double) * col);

  for(i = 0; i < row; i++)
    filter[i] = 1;

  for(i = 0; i < col; i++)
    seed[i] = i;

  for(i = 0; i < row; i++) {
    j = i * col;

    s = 0;
    for(k = 0; k < col-1; k++) {

      switch(front_type) { 
        case SPHERE:
          data[j + k] = EMO_Rand_real1(&rnd, 0, sqrt(1.0 - s));
          s += data[j + k] * data[j+k];
          break;

        case SIMPLEX:
          data[j + k] = EMO_Rand_real1(&rnd, 0, 1.0 - s);
          s += data[j + k];
          break;

        default:
          printf("Unknown Pareto front\n");
          exit(1);
      }
    }

    switch(front_type) { 
      case SPHERE:
        data[j + col - 1] = sqrt(1.0 - s);
        break;

      case SIMPLEX:
        data[j + col - 1] = 1.0 - s;
        break;

      default:
        printf("Unknown Pareto front\n");
        exit(1);
    }

    // zdt3
    //data[j + col - 1] = 1.0 - sqrt(s) - s * sin(10.0 * 3.141516 * s)
  }

  EMO_File_write(data, NULL, row, col, "test1.txt", "%f ");

  i = 0;
  do {
    printf("--------------%d------------\n", i);

    EMO_maxminBound(max, min, data, filter, row, col);

    EMO_File_write(data, filter, row, col, "stdout", "%f "); 
    EMO_vprint(stdout, min, col, "min");
    EMO_vprint(stdout, max, col, "max");

    EMO_normalize(norm, data, filter, row, min, max, col);
    EMO_VPath_prune(&vpath, norm, filter, row, row-1, 1.0);

    if(i % 100 == 0)
      EMO_Plot_run(&plot, data, i, 1); 

    if(i + 1 < run) {
    for(j = 0; j < row; j++)
      if(filter[j] == 0)
        break;

    s = 0;

    EMO_Rand_shuffle(&rnd, seed, col);

    for(k = 0; k < col-1; k++) { 


      switch(front_type) { 
        case SPHERE:
          data[j*col + seed[k]] = EMO_Rand_real1(&rnd, 0, sqrt(1.0 - s));
          s += data[j*col + seed[k]] * data[j*col + seed[k]];
          break;

        case SIMPLEX:
          data[j*col  + seed[k]] = EMO_Rand_real1(&rnd, 0, 1.0 - s);
          s += data[j*col  + seed[k]];
          break;

        default:
          printf("Unknown Pareto front\n");
          exit(1);
      }
    }

      switch(front_type) { 
        case SPHERE:
          data[j*col + seed[k]] = sqrt(1.0 - s);
          break;

        case SIMPLEX:
          data[j*col  + seed[k]] = 1.0 - s;
          break;

        default:
          printf("Unknown Pareto front\n");
          exit(1);
      }


    filter[j] = 1;
    }

    i++;
  } while(i < run);

  EMO_Plot_run(&plot, data, i, 1); 
  EMO_File_write(data, filter, row, col, "test2.txt", "%f ");
  EMO_VPath_write(&vpath, "matrix.out");


  EMO_Rand_free(&rnd);
  EMO_VPath_free(&vpath);
  EMO_Plot_free(&plot);
  free(data);
  free(norm);
  free(filter);
  free(seed);
  free(min);
  free(max);

  return 0;
}

